@@ -3,6 +3,7 @@ require 'net/http'
 require "uri"
 require "etc"
 
+require "librarian/helpers"
 require "librarian/support/abstract_method"
 
 require "librarian/error"
@@ -14,6 +15,7 @@ require "librarian/resolver"
 require "librarian/dsl"
 require "librarian/source"
 require "librarian/version"
+require "librarian/environment/runtime_cache"
 
 module Librarian
   class Environment
@@ -21,6 +23,7 @@ module Librarian
     include Support::AbstractMethod
 
     attr_accessor :ui
+    attr_reader :runtime_cache
 
     abstract_method :specfile_name, :dsl_class, :install_path
 
@@ -29,6 +32,7 @@ module Librarian
       @env = options.fetch(:env) { ENV.to_hash }
       @home = options.fetch(:home) { default_home }
       @project_path = options[:project_path]
+      @runtime_cache = RuntimeCache.new
     end
 
     def logger
@@ -101,8 +105,18 @@ module Librarian
       Lockfile.new(self, nil)
     end
 
-    def resolver
-      Resolver.new(self)
+    def resolver(options = { })
+      Resolver.new(self, resolver_options.merge(options))
+    end
+
+    def resolver_options
+      {
+        :cyclic => resolver_permit_cyclic_reslutions?,
+      }
+    end
+
+    def resolver_permit_cyclic_reslutions?
+      false
     end
 
     def tmp_path
@@ -174,6 +188,10 @@ module Librarian
       no_proxy?(host) ? Net::HTTP : net_http_default_class
     end
 
+    def inspect
+      "#<#{self.class}:0x#{__id__.to_s(16)}>"
+    end
+
   private
 
     def environment
