@@ -1,3 +1,5 @@
+require "pathname"
+
 require "librarian/posix"
 
 module Librarian
@@ -20,16 +22,17 @@ module Librarian
 
           def git_version
             command = %W[#{bin} version --silent]
-            Posix.run!(command).strip =~ /([.\d]+)$/ && $1
+            Posix.run!(command).strip =~ /\Agit version (\d+(\.\d+)*)/ && $1
           end
         end
 
-        attr_accessor :environment, :path
-        private :environment=, :path=
+        attr_accessor :environment, :path, :git_ops_history
+        private :environment=, :path=, :git_ops_history=
 
         def initialize(environment, path)
           self.environment = environment
           self.path = Pathname.new(path)
+          self.git_ops_history = []
         end
 
         def git?
@@ -67,6 +70,13 @@ module Librarian
           run!(command, :chdir => true)
         end
 
+        def has_commit?(sha)
+          command = %W(log -1 --no-color --format=tformat:%H #{sha})
+          run!(command, :chdir => true).strip == sha
+        rescue Posix::CommandFailure => e
+          false
+        end
+
         def checked_out?(sha)
           current_commit_hash == sha
         end
@@ -136,6 +146,8 @@ module Librarian
 
           out = yield
 
+          git_ops_history << command + [{:pwd => pwd}]
+
           unless silent
             if out.size > 0
               out.lines.each do |line|
@@ -147,6 +159,24 @@ module Librarian
           end
 
           out
+
+        rescue Posix::CommandFailure => e
+
+          git_ops_history << command + [{:pwd => pwd}]
+
+          status, stderr = e.status, e.message
+          unless silent
+            debug { "    --- Exited with #{status}" }
+            if stderr.size > 0
+              stderr.lines.each do |line|
+                debug { "    --> #{line}" }
+              end
+            else
+              debug { "    --- No output" }
+            end
+          end
+
+          raise e
         end
 
         def debug(*args, &block)
